angular.module('manageSegments', ['ngResource', 'condition', 'wemServices', 'wemDirectives', 'wemModals', 'picker', 'i18n', 'ui.bootstrap', 'datatables', 'dtPager'])
    .run(['$rootScope', 'i18nService', 'genericModal', 'listService', 'segmentService', 'loadingSpinnerService',
        function ($rootScope, i18nService, genericModal, listService, segmentService, loadingSpinnerService) {
            $rootScope.getCurrentModuleResource = function (resourcePath) {
                return ManagersUtils.getCurrentModuleResource(resourcePath);
            };

            $rootScope.tryDeleteItem = function (segmentOrList, isStaticList, staticListCB, segmentCB) {
                // if the segment we're trying to delete is shared, display a modal to ask confirmation
                if (segmentOrList.scope != ManagersContext.dmfScope) {
                    genericModal.newAlertModal({
                        title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.title" : "serverSettings.manageSegments.delete.sharedWarning.title", segmentOrList.name),
                        bodyMessage: i18nService.message(isStaticList ? "serverSettings.manageLists.delete.sharedWarning.message" : "serverSettings.manageSegments.delete.sharedWarning.message")
                    }, function () {
                        if(isStaticList) {
                            deleteList(segmentOrList, staticListCB);
                        } else {
                            deleteSegment(segmentOrList, segmentCB);
                        }
                    });
                } else {
                    genericModal.newAlertModal({
                        title: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.title" : "serverSettings.manageSegments.delete.title", segmentOrList.name),
                        bodyMessage: i18nService.format(isStaticList ? "serverSettings.manageLists.delete.message" : "serverSettings.manageSegments.delete.message", segmentOrList.name)
                    }, function () {
                        if(isStaticList) {
                            deleteList(segmentOrList, staticListCB);
                        } else {
                            deleteSegment(segmentOrList, segmentCB);
                        }
                    });
                }
            };

            var deleteList = function (list, callback) {
                loadingSpinnerService.show();
                listService.deleteList(list.id).success(function () {
                    loadingSpinnerService.hide();
                    if(callback) {
                        callback();
                    }
                });
            };

            var deleteSegment = function(segment, callback) {
                loadingSpinnerService.show();
                segmentService.deleteSegment(segment.id, true).success(function (data) {
                    loadingSpinnerService.hide();
                    // if data isn't empty, it contains the list of the segments that use the segment we're trying to delete as a condition
                    if (data.segments.length > 0 || data.scorings.length > 0) {
                        // display the list of dependent segments and ask what to do
                        genericModal.newAlertModal({
                            title: i18nService.format("serverSettings.manageSegments.delete.dependentsModal.title", segment.name),
                            bodyTemplateUrl: ManagersContext.localTemplatesPath + "/segments/segment-delete-dependents-modal-body.html",
                            segmentName: segment.name,
                            dependents: data
                        }, function () {
                            loadingSpinnerService.show();
                            segmentService.deleteSegment(segment.id, false).success(function (data) {
                                loadingSpinnerService.hide();
                                if(callback) {
                                    callback();
                                }
                            })
                        });
                    } else {
                        if(callback) {
                            callback();
                        }
                    }
                });
            }
        }])

    .controller('ManageSegmentsListCtrl', ['$scope', '$modal', 'segmentService', 'queryService', 'listService', '$location', 'DTOptionsBuilder', 'DTColumnBuilder', 'filteringService', 'genericModal', 'i18nService', '$compile', '$sce', 'dataTableService',

        function ($scope, $modal, segmentService, queryService, listService, $location, DTOptionsBuilder, DTColumnBuilder, filteringService, genericModal, i18nService, $compile, $sce, dataTableService) {
            $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
            $scope.items = {};

            function serverData(sSource, aoData, fnCallback, oSettings) {
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var start = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    return "metadata." + ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
                }).join(",");

                if($scope.isStaticList){
                    listService.query(listService.constructQuery(length, start, order)).success(function (data) {
                        fnCallback(getResult(draw, data, listService));
                        ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    }).error(function (data, status, headers, config) {
                        fnCallback(getErrorResult());
                    });
                } else {
                    segmentService.query(segmentService.constructQuery(length, start, order)).success(function (data) {
                        fnCallback(getResult(draw, data, segmentService));
                        ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    }).error(function (data, status, headers, config) {
                        fnCallback(getErrorResult());
                    });
                }
            }

            var getResult = function(draw, data, service) {
                angular.forEach(data.list, processItem);

                return {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": data.list
                };
            };

            var getErrorResult = function (draw) {
                $scope.searchError = true;
                return {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
            };

            $scope.dtOptions = dataTableService.generateDataTableOptions({
                pageLength: 25,
                lengthMenu: [ 25, 50, 100 ],
                processing: true,
                serverSide: true,
                sDom: 'lt',
                aaSorting: [],
                createdRow: function (row, data, dataIndex) {
                    console.log("Row is created: "+row);
                    // Recompiling so we can bind Angular directive to the DT
                    $compile(angular.element(row).contents())($scope);
                }
            });

            $scope.dtOptions.withFnServerData(serverData);

            $scope.dtColumns = [
                DTColumnBuilder.newColumn("name").withTitle(i18nService.message($scope.isStaticList ? "serverSettings.manageLists.list.name" : "serverSettings.manageSegments.segmentName"))
                    .renderWith(renderNameCol),
                {"data":'description',"title":i18nService.message("serverSettings.manageWem.description"), "defaultContent": ""},
                DTColumnBuilder.newColumn(null).withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                    .renderWith(actionsHtml).withOption("width","25%").withOption("contentPadding","0")
            ];

            $scope.setDtInstance = function(instance) {
                $scope.dtInstance = instance;
            };

            function actionsHtml(data, type, full, meta) {

                var actions = "<button title=\"" + i18nService.message($scope.isStaticList ? "serverSettings.manageSegments.editList" : "serverSettings.manageSegments.editSegment") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><span class=\"md md-edit\"></span></button> " +
                    "<button title=\"" + i18nService.message("serverSettings.manageSegments.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><span class=\"md md-profiles\"></span></button>";
                if( !_.contains(['contacts', 'leads'], data.id)){
                    actions +=  "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><span class=\"md md-delete\"></span></button>";
                }
                return actions;
            }

            $scope.goToEdit = function(id) {
                $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id );
            };

            $scope.goToProfiles = function(id) {
                $location.path( "/" + ($scope.isStaticList ? "list" : "segment") + "/" +id + "/profiles" );
            };

            function renderNameCol(data, type, full, meta) {
                var result = data + ' <span class="badge green-bg">{{items["' + full.id + '"].count}}</span>';
                if(full.isShared) {
                    result += ' <span class="badge  blue-bg">' + i18nService.message("serverSettings.manageSegments.shared") + '</span>'
                }
                if(!full.enabled) {
                    result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
                }
                return result;
            }

            var processItem = function (itemMetadata) {
                $scope.items[itemMetadata.id] = itemMetadata;
                itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
                itemMetadata.count = 0;
                queryService.getMatchingItemsCountForCondition('profile', {
                    type:"booleanCondition",
                    parameterValues: {
                        subConditions: [
                            {
                                type: "profilePropertyCondition",

                                parameterValues: {
                                    propertyName: $scope.isStaticList ? "systemProperties.lists" : "segments",
                                    comparisonOperator: "equals",
                                    propertyValue: itemMetadata.id
                                }
                            },
                            {
                                type: "profilePropertyCondition",
                                parameterValues: {
                                    propertyName: "mergedWith",
                                    comparisonOperator: "missing"
                                }
                            },
                            {
                                type: "profilePropertyCondition",
                                parameterValues: {
                                    propertyName: "systemProperties.isAnonymousProfile",
                                    comparisonOperator: "missing"
                                }
                            }],
                        operator: "and"
                    }
                }).success(function (data) {
                        itemMetadata.count = data;
                    });
            };

            $scope.deleteItem = function (id) {
                $scope.tryDeleteItem($scope.items[id], $scope.isStaticList, function() {
                    $scope.dtInstance.dataTable.fnDraw()
                }, function() {
                    segmentService.deletedItem = id;
                    $scope.dtInstance.dataTable.fnDraw()
                });
            };
        }])

    .controller('ManageSegmentsCtrl', ['$scope', '$routeParams', '$resource', '$location', '$route', 'segmentService', 'listService', 'currentSegment', 'newSegment', 'definitionService', 'queryService', 'i18nService', 'notificationService', 'loadingSpinnerService',
        function ($scope, $routeParams, $resource, $location, $route, segmentService, listService, currentSegment, newSegment, definitionService, queryService, i18nService, notificationService, loadingSpinnerService) {
            $scope.isStaticList = ManagersContext.isStaticList != undefined ? ManagersContext.isStaticList : false;
            $scope.currentSegment = currentSegment;
            $scope.deleteAllowed = !_.contains(['contacts', 'leads'], currentSegment.metadata.id);
            $scope.conditionTypes = [];
            $scope.sharedSegment = currentSegment.metadata.scope != ManagersContext.dmfScope;
            $scope.newSegment = newSegment || false;
            $scope.cannotShareTooltip = i18nService.message('serverSettings.manageSegments.sharedCannotEdit');

            if(!$scope.isStaticList) {
                var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
                    'get': {
                        method: 'GET',
                        isArray: true
                    }
                });
                var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

                $scope.tags = {};

                $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
                    console.log("manage-segments.js : tags loaded from Apache Unomi.");

                    angular.forEach(value.subTags, function (v) {
                        $scope.tags[v.id] = v
                    });
                    definitionService.fillTags("conditionTypes", $scope);
                    definitionService.fillTags("eventConditionTypes", $scope);
                });

                $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
                    console.log("manage-segment.js: Loaded " + conditionTypes.length + " condition types");
                });
                $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
                    console.log("manage-segment.js: Loaded " + conditionTypes.length + " condition types");
                });

                segmentService.getImpacted(currentSegment.metadata.id).success(function(data){
                    $scope.dependents = data;
                });

                $scope.resetSegment = function ($event) {
                    segmentService.getSegment(currentSegment.metadata.id).success(function (segmentData) {
                        currentSegment = segmentData;
                        $route.reload();
                    });
                };

                $scope.copySegment = function ($event) {
                    segmentService.segmentToCopy = angular.copy(currentSegment);
                    $location.path("/newSegment");
                };

                $scope.hasError = function(propertyName) {
                    return ManagersUtils.hasErrors($scope.segmentForm, $scope.segmentForm, propertyName);
                };
            }

            $scope.saveSegment = function (segmentForm) {
                if(segmentForm.$invalid){
                    segmentForm.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm(segmentForm);
                    if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                        ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                        return;
                    }
                }

                var type = $scope.isStaticList ? 'userList' : 'segment';
                var errorKey = $scope.isStaticList ? "serverSettings.manageLists.create.alreadyExisting" : "serverSettings.manageSegments.create.alreadyExisting";

                var save = function () {
                    if ($scope.isStaticList) {
                        listService.setList($scope.currentSegment, $scope.sharedSegment).success(function () {
                            $location.path("/lists")
                        });
                    } else {
                        segmentService.setSegment($scope.currentSegment, $scope.sharedSegment).success(function () {
                            $location.path("/segments")
                        });
                    }
                };

                loadingSpinnerService.show();
                if ($scope.newSegment) {
                    queryService.createWithCallbackIfDoesNotExist(type, ManagersContext.dmfScope, $scope.currentSegment.metadata.name, null,
                        errorKey, $scope.currentSegment.metadata.name, save, function() {
                            loadingSpinnerService.hide();
                        });
                } else {
                    save();
                }
            };

            $scope.deleteCurrent = function () {
                var id = $scope.currentSegment.metadata.id;
                $scope.tryDeleteItem($scope.currentSegment.metadata, $scope.isStaticList, function() {
                    $location.path('/lists');
                }, function() {
                    segmentService.deletedItem = id;
                    $location.path('/segments');
                });
            };
        }]);

