angular.module('wemDirectives', ['i18n', 'wemServices'])

    .directive('autofocus', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            link: function ($scope, $element) {
                $timeout(function () {
                    $element[0].focus();
                });
            }
        };
    }])

    .directive("pageHead", ['$window', function ($window) {
        return {
            restrict: 'C',
            scope: {
                headerShadow: '@'
            },
            link: function (scope, element, attrs) {
                angular.element($window).bind("scroll", function () {
                    if (this.pageYOffset > (scope.headerShadow ? scope.headerShadow : 0)) {
                        element.addClass("pageHeadShadow");
                    } else {
                        element.removeClass("pageHeadShadow");
                    }
                });
            }
        };
    }])

    .directive('ngEnter', function () {
        return function (scope, element, attrs) {
            element.bind("keydown keypress", function (event) {
                if(event.which === 13) {
                    scope.$apply(function (){
                        scope.$eval(attrs.ngEnter);
                    });

                    event.preventDefault();
                }
            });
        };
    })

    .directive('foldableInput', function () {
        return {
            restrict: 'E',
            scope: {
                model: '=',
                name: '@',
                multivalued: '=',
                required: '=',
                labelKey: '@',
                label: '@',
                altClass: '@',
                type: '@',
                disabled: '=',
                pattern: '@',
                hasError: '='
            },
            transclude:true,
            controller: function ($scope, datepickerConfig, i18nService) {
                if ($scope.type == 'date' && $scope.model != null && typeof $scope.model == "string") {
                    $scope.model = moment($scope.model).toDate();
                }

                $scope.inner = {
                    model: $scope.model
                };

                $scope.tempValue = null;

                var foldedClass;
                if ($scope.altClass) {
                    foldedClass = "foldedLabel " + $scope.altClass;
                } else {
                    foldedClass = "foldedLabel";
                }

                function updateLabel(newValue) {
                    if ((newValue != null && (typeof newValue == "number" || newValue != '')) && !$scope.multivalued) {
                        $scope.labelClass = foldedClass;
                    } else {
                        $scope.labelClass = "";
                    }
                }

                $scope.focusInput = function (event) {
                    $scope.labelClass = foldedClass;
                    $scope.focused = true;
                    if ($scope.type == "date") {
                        $scope.datepicker.datePickerOpen = true;
                        event.target.blur()
                    }
                };

                $scope.blurInput = function (event) {
                    //Callback for blur event defined on the parent Scope
                    if ($scope.type != "date" && $scope.$parent["blurCallback"]) {
                        $scope.$parent["blurCallback"](event);
                    }
                    updateLabel(event.target.value);
                    $scope.focused = false;
                };

                $scope.$watch("inner.model", function(newVal) {
                    $scope.model = newVal;
                });

                $scope.$watch("model", function (newValue, oldValue) {
                    $scope.inner.model = newValue;
                    if (!$scope.focused) {
                        updateLabel(newValue);
                    }
                });

                updateLabel($scope.model);
                $scope.focused = false;

                if ($scope.type == 'date') {
                    datepickerConfig.startingDay = 1;
                    datepickerConfig.minMode = "day";
                    datepickerConfig.datepickerMode = "day";
                    datepickerConfig.showWeeks = false;

                    $scope.openDatePopup = function ($event, selector) {
                        $event.preventDefault();
                        $event.stopPropagation();

                        $scope.datepicker.datePickerOpen = true;
                    };
                    $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
                    $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
                    $scope.datepicker = {
                        datePickerOpen: false
                    };
                }
                
                $scope.onDeleteMultiValue = function (property, value) {
                    return _.reject(property, function(val){ return val === value; });
                };

                $scope.onAddMultiValue = function (property, newValue) {
                    if(!property){
                        property = [];
                    }
                    if(!Array.isArray(property)){
                        property = Array.of(property);
                    }
                    if(!_.contains(property, newValue)) {
                        property.push(angular.copy(newValue));
                    }
                    
                    return property;
                };


            },
            link: function (scope, element, attrs) {
                if (scope.labelKey && ManagersContext.i18nLabels && ManagersContext.i18nLabels[scope.labelKey]) {
                    scope.label = ManagersContext.i18nLabels[scope.labelKey];
                }
                if (scope.required) {
                    scope.label = scope.label + " (" + ManagersContext.i18nLabels['label.required'] + ")";
                }

                element.find("label").text(scope.label).click(function () {
                    // need for IE that doesn't support  pointer-events: none; in CSS
                    if (attrs.type == "textarea") {
                        element.find("textarea").focus();
                    } else {
                        element.find("input").focus();
                    }
                });

                if (attrs.type == "textarea") {
                    scope.templateUrl = ManagersContext.localTemplatesPath + "/foldable-input/textarea.html";
                } else if(attrs.type == "date") {
                    scope.templateUrl =  ManagersContext.localTemplatesPath + "/foldable-input/date.html";
                } else {
                    if(scope.multivalued){
                        scope.templateUrl =  ManagersContext.localTemplatesPath + "/foldable-input/multipleInput.html";
                    }else{
                        scope.templateUrl =  ManagersContext.localTemplatesPath + "/foldable-input/generic.html";
                    }
                }
            },
            templateUrl:  ManagersContext.localTemplatesPath + "/foldable-input/base.html"
        };
    })

    .directive('lettersOrNumbersOnly', [
        function() {
            var link = function ($scope, $element, $attrs, ctrl) {
                var validate = function (viewValue) {
                    ctrl.$setValidity('lettersOrNumbersOnly', new RegExp("^[A-Za-z0-9]+$").test(viewValue));
                    return viewValue;
                };
                ctrl.$parsers.unshift(validate);
                ctrl.$formatters.push(validate);

                $attrs.$observe('lettersOrNumbersOnly', function (comparisonModel) {
                    // Whenever the comparison model changes we'll re-validate
                    return validate(ctrl.$viewValue);
                });

            };
            return {
                require: 'ngModel',
                link: link
            };
        }
    ])

    .directive('lowerThanNumber', [
        function () {
            var link = function ($scope, $element, $attrs, ctrl) {
                var validate = function (viewValue) {
                    var comparisonModel = $attrs.lowerThanNumber;
                    var view = parseInt(viewValue, 10);
                    var comparison = parseInt(comparisonModel, 10);

                    if (view && comparison) {
                        // It's valid because we have nothing to compare against
                        ctrl.$setValidity('lowerThan', view < comparison);
                    } else {
                        ctrl.$setValidity('lowerThan', true);
                    }

                    return viewValue;
                };

                ctrl.$parsers.unshift(validate);
                ctrl.$formatters.push(validate);

                $attrs.$observe('lowerThanNumber', function (comparisonModel) {
                    // Whenever the comparison model changes we'll re-validate
                    return validate(ctrl.$viewValue);
                });

            };

            return {
                require: 'ngModel',
                link: link
            };

        }
    ])

    .directive('loadingSpinner', function (loadingSpinnerService) {
        return {
            restrict: 'E',
            scope: {

            },
            template: [
                '<div ng-show="show" class="loadingSpinner">',
                '  <div class="loadingSpinnerOverlay"></div>',
                '  <svg class="loadingSpinnerCircular" height="50" width="50">',
                '    <circle class="loadingSpinnerPath" cx="25" cy="25" r="20" fill="none" stroke-width="6" stroke-miterlimit="10" />',
                '  </svg>',
                '</div>'
            ].join(''),
            controller: function ($scope) {
                $scope.show  = false;

                loadingSpinnerService.register({
                    'show': function() {
                        $scope.show = true;
                    },
                    'hide': function() {
                        $scope.show = false;
                    }
                });
            }
        };
    })

    .directive('prefixModelByProperties', function () {
        return {
            require: 'ngModel',
            restrict: 'A',
            scope: {ngModel: '='},
            link: function (scope, element, attr, ngModelController) {
                ngModelController.$formatters.unshift(function (valueFromModel) {
                    // what you return here will be passed to the text field
                    if(valueFromModel && valueFromModel.lastIndexOf("properties.", 0) === 0) {
                        return valueFromModel.substr("properties.".length);
                    }
                    return valueFromModel;
                });

                ngModelController.$parsers.push(function (valueFromInput) {
                    // put the inverse logic, to transform formatted data into model data
                    // what you return here, will be stored in the $scope
                    return "properties." + valueFromInput;
                });
            }
        };
    });
