angular.module('wemServices', ['underscore', 'ui-notification', 'i18n', 'datatables', 'datatables.bootstrap'])

    .config(['$httpProvider', 'NotificationProvider', function ($httpProvider, NotificationProvider) {
        $httpProvider.defaults.headers.common['Accept-Language'] = ManagersContext.jahiaUILocale;

        $httpProvider.interceptors.push(function ($q,i18nService) {
            return {
                'responseError': function (rejection) {
                    if (ManagersUtils.notificationProvider && rejection.status != 404) {
                        if (rejection.data && rejection.data.error) {
                            ManagersUtils.notificationProvider.error("Error : " + rejection.data.error);
                        } else if (rejection.data && rejection.data != null) {
                            ManagersUtils.notificationProvider.error("Error : " + rejection.data);
                        } else {
                            ManagersUtils.notificationProvider.error("Error : " + rejection.statusText);
                        }
                    }
                    if (rejection.config.url.match(/templates\/condition/)) {
                        rejection.data = i18nService.message("condition.unknown");
                        return rejection;
                    }

                    return $q.reject(rejection);
                }
            };
        });

        NotificationProvider.setOptions({
            delay: 5000,
            startTop: 20,
            startRight: 20,
            verticalSpacing: 20,
            horizontalSpacing: 20,
            positionX: 'right',
            positionY: 'bottom'
        });
    }])

    .run(['Notification', '$rootScope', '$window', '$location',
        function (Notification, $rootScope, $window, $location) {
            ManagersUtils.notificationProvider = Notification;

            $rootScope.$on('$locationChangeSuccess', function (event) {
                $window.parent.location.hash = "#" + $location.path();
            });
        }])

    .service('notificationService', ['Notification', function (Notification) {
        return Notification;
    }])

    .service('loadingSpinnerService', function () {
        this.register = function (spinner) {
            this.spinner = spinner;
        };

        this.show = function () {
            if (this.spinner) {
                this.spinner.show();
            }
        };

        this.hide = function () {
            if (this.spinner) {
                this.spinner.hide();
            }
        }
    })

    .service('validationUtilsService', ['loadingSpinnerService', function (loadingSpinnerService) {
        this.getFormInPage = function ($scope, formId, path, callback) {
            var id = "formFrame"+Math.random().toString(16).substring(2);
            var tries = 5;
            console.log("creating frame " + id);
            var $iframe = $("<iframe id='"+id+"'></iframe>");
            $iframe.css("display", "none");
            $("body").append($iframe);
            loadingSpinnerService.show();
            $iframe.ready(function () {
                console.log("iframe "+id+" loaded")
                var f = function () {
                    $scope.$apply(function() {
                        var _form = $iframe.contents().find("form[name='" + formId + "']");
                        if (!_form || _form.length == 0) {
                            _form = $iframe.contents().find("#" + formId);
                        }
                        if ((_form && _form.length > 0 && _form.find(":input").length > 0) || tries == 0) {
                            console.log("found form : "+ _form.length);
                            $("#"+id).remove();
                            loadingSpinnerService.hide();
                            return callback(_form);
                        } else {
                            tries --;
                            console.log("not found, retrying : " +tries);
                            window.setTimeout(f, 1000);
                        }
                    });
                };
                window.setTimeout(f, 1000);
            });
            $iframe.attr("src", ManagersContext.baseLive + path + ".html?skipWem=true");
        };
    }])

    .service('filteringService', function (_) {
        this.filterItems = function (items, service) {
            if (service.deletedItem) {
                items = _.filter(items, function (item) {
                    return item.id != service.deletedItem
                });
            }
            if (service.newItem) {
                var found = false;
                items = _.map(items, function (item) {
                    if (item.id == service.newItem.metadata.id) {
                        found = true;
                        return service.newItem.metadata;
                    } else {
                        return item;
                    }
                });
                if (!found) {
                    items.push(service.newItem.metadata);
                }
            }
            delete service.newItem;
            delete service.deletedItem;

            return items;
        };
    })

    .service('goalService', ['$http', 'loadingSpinnerService', function ($http, loadingSpinnerService) {
        var addToListRulePrefix = "addToListRULE_";
        var goalsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'goals/';

        this.getAddToListRulePrefix = function(){
            return addToListRulePrefix;
        };

        this.getAll = function () {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.getAllForSite = function () {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "missing"
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.getAllForCampaign = function (campaign) {
            return $http.post(goalsContext + "query", {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaign
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.localToServerGoalRule = function(goal, lists){
            if(goal) {
                var _goalRule = {
                    "metadata": {
                        "id": addToListRulePrefix+goal.metadata.id,
                        "name": goal.metadata.name,
                        "description": goal.metadata.description,
                        "scope": goal.metadata.scope
                    },
                    "condition": {
                        type: "booleanCondition",
                        parameterValues: {
                            subConditions: [
                                {
                                    type: "eventPropertyCondition",
                                    parameterValues: {
                                        propertyName: "target.itemId",
                                        comparisonOperator: "equals",
                                        propertyValue: goal.metadata.id+'TargetEvent'
                                    }
                                },
                                {
                                    type: "eventTypeCondition",
                                    parameterValues: {
                                        eventTypeId: "ruleFired"
                                    }
                                }
                            ],
                            operator: "and"
                        }
                    },

                    "actions": []
                };

                //lists
                angular.forEach(lists, function (value, key) {
                    _goalRule.actions.push({
                        "parameterValues": {
                            "setPropertyName": "systemProperties.lists",
                            "setPropertyValue": value,
                            "setPropertyStrategy": "addValue"
                        },
                        "type": "setPropertyAction"
                    });
                });

                return _goalRule;
            }

            return null;
        };

        this.setGoal = function (goal) {
            this.newItem = goal;
            return $http.post(goalsContext, goal);
        };

        this.getGoal = function (id) {
            return $http.get(goalsContext + id);
        };

        this.getGoalReport = function (id) {
            var uri = goalsContext + id + "/report";
            return $http.get(uri);
        };

        this.getGoalReportAggregated = function (id, query) {
            var uri = goalsContext + id + "/report";
            return $http.post(uri, query);
        };

        this.deleteGoal = function (id) {
            this.deletedItem = id;
            return $http.delete(goalsContext + id);
        };
    }])

    .service('ruleService', ['$http', function ($http) {
        var rulesContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/';

        this.getAll = function () {
            return $http.post(rulesContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.setRule = function (rule) {
            return $http.post(rulesContext, rule);
        };

        this.getRule = function (id) {
            return $http.get(rulesContext + id);
        };

        this.deleteRule = function (id) {
            return $http.delete(rulesContext + id);
        };
    }])

    .service('segmentService', ['$http', 'jcrService', function ($http, jcrService) {
        var segmentsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'segments/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedItem;
            }

            return query;
        };

        this.getAll = function () {
            return $http.get(segmentsContext);
        };

        this.getImpacted = function (segmentId) {
            return $http.get(segmentsContext + segmentId + "/impacted");
        };

        this.query = function (query) {
            return $http.post(segmentsContext + "query", query);
        };

        this._getSubConditionsOfType = function (rootCondition, list, conditionType) {
            var instance = this;
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    instance._getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        };

        this.addDownloadMixin = function(condition) {
            var instance = this;
            pastEventConditions = this._getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = instance._getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).success(function (data) {
                        // mixin added
                    });
                })
            });
        };

        this.setSegment = function (segment, shared) {
            this.addDownloadMixin(segment.condition);
            if (shared) {
                segment.metadata.scope = 'systemscope';
            }
            this.newItem = segment;
            return $http.post(segmentsContext, segment);
        };

        this.getSegment = function (id) {
            return $http.get(segmentsContext + id);
        };

        this.getSegmentCount = function (id) {
            return $http.get(segmentsContext + id + "/count");
        };

        this.deleteSegment = function (id, validate) {
            var uri = segmentsContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        }
    }])

    .service('scoringService', ['$http', 'jcrService', function ($http, jcrService) {
        var scoringContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'scoring/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedItem;
            }

            return query;
        };

        this.getAll = function () {
            return $http.get(scoringContext);
        };

        this.query = function (query) {
            return $http.post(scoringContext + "query", query);
        };

        this._getSubConditionsOfType = function (rootCondition, list, conditionType) {
            var instance = this;
            if (rootCondition.type == "booleanCondition" && rootCondition.parameterValues && rootCondition.parameterValues.subConditions) {
                rootCondition.parameterValues.subConditions.forEach(function(subCondition) {
                    instance._getSubConditionsOfType(subCondition, list, conditionType);
                })
            } else if (rootCondition.type == conditionType) {
                list.push(rootCondition);
            }

            return list;
        };

        this.addDownloadMixin = function(condition) {
            var instance = this;
            pastEventConditions = this._getSubConditionsOfType(condition, [], "pastEventCondition");
            pastEventConditions.forEach(function (pastEventCondition) {
                var downloadConditions = instance._getSubConditionsOfType(pastEventCondition.parameterValues.eventCondition, [], "downloadEventCondition");
                downloadConditions.forEach(function (downloadCondition) {
                    jcrService.addMixin("live", null, downloadCondition.parameterValues.filePath, "wemmix:trackDownloadedFile", {}, true).success(function (data) {
                        // mixin added
                    });
                });
            });
        };

        this.setScoring = function (scoring, shared) {
            var instance = this;
            scoring.elements.forEach(function (element) {
                instance.addDownloadMixin(element.condition);
            });
            if (shared) {
                scoring.metadata.scope = 'systemscope';
            }
            this.newItem = scoring;
            return $http.post(scoringContext, scoring);
        };

        this.getScoring = function (id) {
            return $http.get(scoringContext + id);
        };

        this.deleteScoring = function (id, validate) {
            var uri = scoringContext + id;
            if (validate) {
                uri += '?validate=true';
            }
            return $http.delete(uri);
        };

        this.getImpacted = function (id) {
            return $http.get(scoringContext + id + "/impacted");
        };

        this.buildUpdateBatchRemoveScoring = function(scoreId){
            var baseCondition = {
                parameterValues: {
                    comparisonOperator: "exists",
                    propertyName: "scores."+scoreId
                },
                type: "profilePropertyCondition"
            };
            return {
                condition: baseCondition,
                propertyName: "systemProperties.notificationAck.scoring."+scoreId,
                propertyValue: null,
                strategy: "remove"
            };  
        };

        this.localToServerScoringRule = function(scoringPlan, targetEmail, threshold){
            var sendEmailRulePrefix = "sendEmailRulePrefix_";
            if(scoringPlan && targetEmail) {
                var _scoringRule = {
                    "metadata": {
                        "id": sendEmailRulePrefix+scoringPlan.metadata.id,
                        "name": scoringPlan.metadata.name,
                        "description": scoringPlan.metadata.description,
                        "scope": "systemscope"
                    },
                    "raiseEventOnlyOnceForProfile": true,
                    "condition": {
                        type: "booleanCondition",
                        parameterValues: {
                            subConditions: [
                                {
                                    type: "eventPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "profile.scores."+scoringPlan.metadata.id,
                                        "propertyValueInteger": threshold,
                                        "comparisonOperator": "greaterThanOrEqualTo"
                                    }
                                },
                                {
                                    type: "eventTypeCondition",
                                    parameterValues: {
                                        eventTypeId: "profileUpdated"
                                    }
                                }
                            ],
                            operator: "and"
                        }
                    },
                    "actions": [
                        {
                            "parameterValues": {
                                "notificationType": "scoring",
                                "notificationTypeId": scoringPlan.metadata.id,
                                "notifyOncePerProfile": true,
                                "from": "test@jahia.com",
                                "to": targetEmail,
                                "subject": "Scoring plan ["+ scoringPlan.metadata.name + "] : New profile matching the conditions",
                                "template": "New profile matching scoring plan [" + scoringPlan.metadata.name + "] conditions. " +
                                                "<if(!profile.properties.(\"firstName\"))> <if(!profile.properties.(\"lastName\"))> Unamed profile <endif> <endif> " +
                                                "<if(profile.properties.(\"firstName\"))> <profile.properties.(\"firstName\")> <endif> " +
                                                "<if(profile.properties.(\"lastName\"))> <profile.properties.(\"lastName\")> <endif> has a score value: <profile.scores.(\"" + scoringPlan.metadata.id + "\")>."
                            },
                            "type": "sendMailAction"
                        }
                    ]
                };

                return _scoringRule;
            }

            return null;
        };
    }])
    .filter('unique', function() {
        return function(collection, keyname) {
            var output = [],
                keys = [];

            angular.forEach(collection, function(item) {
                var key = item[keyname];
                if(keys.indexOf(key) === -1) {
                    keys.push(key);
                    output.push(item);
                }
            });

            return output;
        };
    })

    .service('listService', ['$http', function ($http) {
        var listsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'lists/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "parameterValues": {
                                    "subConditions": [
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": ManagersContext.dmfScope
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "metadata.scope",
                                                "comparisonOperator": "equals",
                                                "propertyValue": "systemscope"
                                            }
                                        }
                                    ],
                                    "operator": "or"
                                },
                                "type": "booleanCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null,
                "forceRefresh": false
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedItem
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedItem;
            }

            return query;
        };

        this.getAll = function () {
            return $http.get(listsContext);
        };

        this.query = function (query) {
            var query = ManagersUtils.getQueryOrDefault(query);
            return $http.post(listsContext + "query", query);
        };

        this.setList = function (list, shared) {
            if (shared) {
                list.metadata.scope = 'systemscope';
            }
            this.newItem = list;
            return $http.post(listsContext, list);
        };

        this.getList = function (id) {
            return $http.get(listsContext + id);
        };

        this.deleteList = function (id) {
            this.deletedItem = id;
            return $http.delete(listsContext + id);
        };
    }])

    .service('queryService', ['$http', 'i18nService', 'notificationService', function ($http, i18nService, notificationService) {
        var queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'query/';

        this.query = function (type, property, aggregate) {
            if (!aggregate){
                return $http.get(queryContext + type + '/' + property);
            }
            return $http.post(queryContext + type + '/' + property, aggregate);
        };

        this.getMetricsForCondition = function (type, property, metrics, condition) {
            return $http.post(queryContext + type + '/' + property + '/' + ($.isArray(metrics) ? metrics.join('/') : metrics), condition);
        };

        this.getMatchingItemsCountForCondition = function (type, condition, validate) {
            return $http.post(queryContext + type + '/count' + (validate != undefined ? ("?validate="+validate) : ""), condition);
        };

        this.createWithCallbackIfDoesNotExist = function (itemType, scope, name, typeId, errorMessageKey, errorMessageParams, callback, failCallback) {

            var condition = {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.scope",
                                comparisonOperator: "equals",
                                propertyValue: scope ? scope : ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "metadata.name",
                                comparisonOperator: "equals",
                                propertyValue: name
                            }
                        }
                    ],
                    operator: "and"
                }
            };

            if (typeId) {
                condition.parameterValues.subConditions.push({
                    type: "sessionPropertyCondition",
                    parameterValues: {
                        propertyName: "metadata.tags",
                        comparisonOperator: "contains",
                        propertyValue: typeId
                    }
                });
            }

            this.getMatchingItemsCountForCondition(itemType, condition).success(function (data) {
                // if we have matching elements, display an error
                if (data > 0) {
                    if (failCallback) {
                        failCallback();
                    }
                    notificationService.error(errorMessageParams ? i18nService.format(errorMessageKey, errorMessageParams) : i18nService.message(errorMessageKey));
                } else {
                    // otherwise perform the provide callback
                    callback();
                }
            });

        };

        // todo: move to campaignService?
        this.nbOfVisits = function (campaignId, startDate, endDate, callback) {
            this.getMetricsForCondition("session", "profile.properties.nbOfVisits", ["sum", "avg"], {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "scope",
                                "comparisonOperator": "equals",
                                "propertyValue": ManagersContext.dmfScope
                            }
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "systemProperties.campaigns." + campaignId + "Engaged",
                                "comparisonOperator": "between",
                                "propertyValuesDate": [startDate, endDate]
                            }
                        }
                    ]
                }
            }).success(callback);
        };
    }])

    .service('definitionService', ['$http', 'i18nService', function ($http, i18nService) {
        var definitionsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'definitions/';

        this.getTag = function (tag) {
            return $http.get(definitionsContext + 'tags/' + tag)
        };

        this.getConditionTypes = function (tag) {
            return $http.get(definitionsContext + 'conditions')
        };

        this.getConditionType = function (conditionType) {
            return $http.get(definitionsContext + 'conditions/' + conditionType)
        };

        this.getConditionTypeByTag = function (tag) {
            return $http.get(definitionsContext + 'conditions/tags/' + tag)
        };
        
        this.fillTags = function (arrayName, $scope) {
            $scope.$watch(arrayName + ".length", function (newVal, oldVal, scope) {
                angular.forEach($scope[arrayName], function (condition) {
                    if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[condition.id]) {
                        condition.profileTag = ManagersContext.conditionsTags[condition.id];
                    } else {
                        angular.forEach(condition.tags, function (tag) {
                            if ($scope.tags[tag]) {
                                condition.profileTag = tag;
                            }
                        });
                    }
                });
            });
        };
    }])

    .service('profileService', ['$http', 'queryService', function ($http, queryService) {
        var profilesContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'profiles/';
        var personasContext = profilesContext + 'personas/';

        var getContextFor = function (profileType) {
            return profileType == 'personas' ? personasContext : profilesContext;
        };

        this.search = function (query, profileType) {
            return $http.post(getContextFor(profileType) + "search", query);
        };

        this.searchSessions = function (query, profileType) {
            return $http.post(getContextFor(profileType) + 'search/sessions', query);
        };

        this.exportUsingForm = function (query, profileType, method) {
            ManagersUtils.downloadForm(getContextFor(profileType) + "export", query, method);
        };

        this.getProfile = function (id, profileType) {
            return $http.get(getContextFor(profileType) + id);
        };

        this.saveProfile = function (profile, profileType) {
            // remove empty props
            var cleanedProp = {};
            angular.forEach(profile.properties, function (value, key) {
                if (value) {
                    cleanedProp[key] = value;
                } else {
                    cleanedProp[key] = null;
                }
            });
            profile.properties = cleanedProp;
            return $http.post(getContextFor(profileType), profile);
        };

        this.deleteProfile = function (id, profileType) {
            return $http.delete(getContextFor(profileType) + id);
        };

        this.newProfile = function (id, profileType) {
            return $http.put(getContextFor(profileType) + id);
        };

        this.batchProfilesUpdate = function (batchUpdate) {
            return $http.post(profilesContext + 'batchProfilesUpdate', batchUpdate)
        };

        this.getSessions = function (id, profileType, offset, numberOfSessions, sort) {
            return $http.get(getContextFor(profileType) + id + '/sessions?offset=' + offset + '&size=' + numberOfSessions + '&sort=' + sort);
        };

        this.getPropertyTypes = function () {
            return $http.get(profilesContext + 'properties');
        };

        this.createPropertyType = function (propertyType) {
            return $http.post(profilesContext + 'properties', propertyType);
        };

        this.deletePropertyType = function (propertyId) {
            return $http.delete(profilesContext + 'properties/' + propertyId);
        };

        this.getPropertyTypesByTarget = function (target) {
            return $http.get(profilesContext + 'properties/' + target)
        };
        this.getPropertyTypesByTag = function (tag, recursive) {
            if (recursive) {
                return $http.get(profilesContext + 'properties/tags/' + tag + '?recursive=true')
            } else {
                return $http.get(profilesContext + 'properties/tags/' + tag)
            }
        };

        this.getCount = function () {
            return queryService.getMatchingItemsCountForCondition('profile',this.constructCondition(null,null,null, true,'profile'), false);
        };


        this.constructCondition = function (baseCondition, searchCondition, deletedProfileId, excludeMergedAndAnonymous, profileType) {
            var subConditions = [];
            if (baseCondition) {
                subConditions.push(baseCondition);
            }
            if (excludeMergedAndAnonymous && profileType != "personas") {
                // we do not want the merged profiles:
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "missing",
                        propertyName: "mergedWith"
                    },
                    type: "profilePropertyCondition"
                });
                subConditions.push({
                    parameterValues: {
                        propertyName: "systemProperties.isAnonymousProfile",
                        comparisonOperator: "missing"
                    },
                    type: "profilePropertyCondition"
                });
            }
            if (searchCondition && searchCondition.type) {
                if (searchCondition.type == "booleanCondition"
                    && searchCondition.parameterValues.operator
                    && searchCondition.parameterValues.operator == "and"
                    && searchCondition.parameterValues.subConditions) {
                    subConditions = subConditions.concat(searchCondition.parameterValues.subConditions);
                } else {
                    subConditions.push(searchCondition);
                }
            }
            if (deletedProfileId) {
                subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: deletedProfileId
                    },
                    type: "profilePropertyCondition"
                });
            }

            if (subConditions.length == 0) {
                return null;
            } else if (subConditions.length == 1) {
                return subConditions[0];
            } else {
                return {
                    parameterValues: {
                        operator: "and",
                        subConditions: subConditions
                    },
                    type: "booleanCondition"
                };
            }
        };

        this.constructQuery = function (baseCondition, searchCondition, deletedProfileId, text, offset, limit, orderby, profileType) {
            return {
                "condition": this.constructCondition(baseCondition, searchCondition, deletedProfileId, true, profileType),
                "limit": limit ? limit : 50,
                "offset": offset ? offset : 0,
                "sortby": orderby != undefined ? orderby : null,
                "text": text != undefined ? text : null
            };
        };

        this.constructQueryOnCurrentSelectedTab = function (text, offset, limit, orderby) {
            if (this.selectedTab) {
                return {
                    "condition": this.constructCondition(this.selectedTab.baseCondition, this.selectedTab.searchCondition, this.selectedTab.deletedProfileId, true, this.selectedTab.type),
                    "limit": limit ? limit : -1,
                    "offset": offset ? offset : 0,
                    "sortby": orderby != undefined ? orderby : null,
                    "text": text != undefined ? text : null
                };
            }
        };

    }])

    .service('jcrService', ['$http', function ($http) {
        this.buildURL = function (workspace, locale, type, endUrl) {
            return [ManagersContext.jcrRestAPIBase,
                    workspace ? workspace : "default",
                    locale ? locale : ManagersContext.currentSiteDefaultLanguage,
                    type ? type : "nodes"].join("/") + endUrl;
        };

        this.addMixin = function (workspace, locale, nodeIdOrPath, mixin, mixinProperties, byPath) {
            var apiType = byPath ? "paths" : "nodes";
            nodeIdOrPath = byPath ? nodeIdOrPath : "/" + nodeIdOrPath;
            var url = this.buildURL(workspace, locale, apiType, nodeIdOrPath) + "/mixins/" + mixin.replace(":", "__");
            return mixinProperties ? $http.put(url, mixinProperties) : $http.put(url);
        };

        this.doGetOnPath = function (workspace, locale, path) {
            return $http.get(this.buildURL(workspace, locale, "paths", path));
        };

        this.doGetOnPathSync = function (workspace, locale, path) {
            var request = new XMLHttpRequest();
            request.open('GET', this.buildURL(workspace, locale, "paths", path), false);
            request.send(null);

            return request.status === 200;
        }
    }])

    .service('campaignService', ['$http', 'i18nService', function ($http, i18nService) {
        var campaignsContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'campaigns/';

        this.constructQuery = function (length, start, order) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (this.deletedCampaign) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedCampaign
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedCampaign
            }

            return query;
        };

        this.getAll = function () {
            return $http.post(campaignsContext + "query", {
                "condition": {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "equals",
                        "propertyValue": ManagersContext.dmfScope
                    }
                },
                "limit": 50,
                "offset": 0,
                "sortby": null
            });
        };

        this.queryDetails = function (query) {
            return $http.post(campaignsContext + "query/detailed", query);
        };

        this.setCampaign = function (campaign) {
            return $http.post(campaignsContext, campaign);
        };

        this.setCampaignEvent = function (campaignEvent) {
            this.newEvent = campaignEvent;
            return $http.post(campaignsContext + "event", campaignEvent);
        };

        this.getCampaign = function (id) {
            return $http.get(campaignsContext + id);
        };

        this.getCampaignDetail = function (id) {
            return $http.get(campaignsContext + id + "/detailed");
        };

        this.getCampaignType = function (campaign) {
            return campaign.metadata.tags[0];
        };

        this.getLocalizedCampaignType = function (campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType);
        };

        this.getLocalizedCampaignParam = function (campaignType) {
            return i18nService.message('serverSettings.manageCampaigns.types.' + campaignType + '.paramName');
        };

        this.getTypeSpecificCampaignParameter = function (campaign) {
            var campaignType = this.getCampaignType(campaign);
            switch (campaignType) {
                case 'landing':
                    return campaign.entryCondition.parameterValues.subConditions[1].parameterValues.pagePath;
                case 'referrer':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
                case 'parameter':
                    return campaign.entryCondition.parameterValues.subConditions[2].parameterValues.propertyValue;
            }
        };

        this.getCampaignEvents = function (campaignId, length, start, order, additionalCondition) {
            var query = {
                "condition": {
                    "parameterValues": {
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "campaignId",
                                    "comparisonOperator": "equals",
                                    "propertyValue": campaignId
                                }
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                },
                "limit": length ? length : 50,
                "offset": start ? start : 0,
                "sortby": order ? order : null
            };

            if (additionalCondition) {
                query.condition.parameterValues.subConditions.push(additionalCondition);
            }

            if (this.newEvent) {
                query.forceRefresh = true;
                delete this.newEvent;
            }

            if (this.deletedEvent) {
                query.condition.parameterValues.subConditions.push({
                    parameterValues: {
                        comparisonOperator: "notEquals",
                        propertyName: "itemId",
                        propertyValue: this.deletedEvent
                    },
                    type: "sessionPropertyCondition"
                });
                delete this.deletedEvent;
            }
            return $http.post(campaignsContext + "events/query", query);
        };

        this.deleteCampaign = function (id) {
            this.deletedCampaign = id;
            return $http.delete(campaignsContext + id);
        };

        this.deleteCampaignEvent = function (eventId) {
            this.deletedEvent = eventId;
            return $http.delete(campaignsContext + "event/" + eventId);
        }
    }])

    .service('formMappingService', ['$http', 'i18nService', 'loadingSpinnerService', function ($http, i18nService, loadingSpinnerService) {
        var formMappingContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'rules/';
        var openMappedProp = "(";
        var setPropertyNamePrefix = "properties" + openMappedProp;
        var setPropertyValuePrefix = "eventProperty::properties" + openMappedProp;
        var endPrefix = ")";
        this.save = function (formMapping) {
            this.newItem = formMapping;
            return $http.post(formMappingContext, formMapping);
        };

        this.getAll = function (offset, limit, sortby) {

            var query = {
                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.tags",
                                    "comparisonOperator": "equals",
                                    "propertyValue": "formMappingRule"
                                }
                            },
                            {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "metadata.scope",
                                    "comparisonOperator": "equals",
                                    "propertyValue": ManagersContext.dmfScope
                                }
                            }
                        ]
                    }
                },
                "offset": offset,
                "limit": limit,
                "sortby": sortby
            };

            if (this.newItem) {
                query.forceRefresh = true;
                delete this.newItem;
            }

            if (this.deletedItem) {
                query.condition = {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": [
                            query.condition,
                            {
                                parameterValues: {
                                    comparisonOperator: "notEquals",
                                    propertyName: "itemId",
                                    propertyValue: this.deletedItem
                                },
                                type: "sessionPropertyCondition"
                            }
                        ]
                    }
                };
                delete this.deletedItem;
            }

            return $http.post(formMappingContext + "query", query);
        };

        this.get = function (id) {
            return $http.get(formMappingContext + id);
        };

        this.serverToLocalFormMapping = function (formMapping) {
            var _formMapping = {
                name: formMapping.metadata.name,
                description: formMapping.metadata.description,
                id: formMapping.metadata.id,
                scope: formMapping.metadata.scope,
                additionalPaths: [],
                tempActions: formMapping.actions,
                actions: {},
                lists: []
            };

            _formMapping.formId = _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type == 'formEventCondition';
            }).parameterValues.formId;

            _.find(formMapping.condition.parameterValues.subConditions, function (entry) {
                return entry.type != 'formEventCondition';
            }).parameterValues.subConditions.forEach(function (entry) {
                    if (!_formMapping.primaryPath) {
                        _formMapping.primaryPath = entry.parameterValues.path;
                    } else {
                        _formMapping.additionalPaths.push(entry.parameterValues.path);
                    }
                });

            return _formMapping;
        };

        this.serverToLocalFormMappingActions = function (formMapping, propertyNameChoiceListValuesIndexById) {

            function extractNameFromPrefixedName(prefixedName, prefix) {
                var extracted;
                if (prefixedName.indexOf(endPrefix) > 0) {
                    extracted = prefixedName.substring(prefix.length, prefixedName.length - endPrefix.length);
                } else {
                    // we have an old style property, so remove the old style prefix which wasn't using indexed properties
                    // prefix contains the opening of the mapped prop ((') so remove it and add + 1 for the . that was there before
                    extracted = prefixedName.substr(prefix.length - openMappedProp.length + 1);
                }
                return extracted;
            }

            formMapping.tempActions.forEach(function (action) {
                if (action.parameterValues.setPropertyName == "systemProperties.lists" && action.parameterValues.setPropertyValue) {
                    formMapping.lists.push(action.parameterValues.setPropertyValue);
                } else {
                    var propertyName = extractNameFromPrefixedName(action.parameterValues.setPropertyName, setPropertyNamePrefix);
                    var fieldName = action.parameterValues.setPropertyValue ? extractNameFromPrefixedName(action.parameterValues.setPropertyValue, setPropertyValuePrefix) : undefined;
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueInteger ? extractNameFromPrefixedName(action.parameterValues.setPropertyValueInteger, setPropertyValuePrefix) : undefined;
                    }
                    if (!fieldName) {
                        fieldName = action.parameterValues.setPropertyValueMultiple ? extractNameFromPrefixedName(action.parameterValues.setPropertyValueMultiple, setPropertyValuePrefix) : undefined;
                    }
                    var strategy = action.parameterValues.setPropertyStrategy ? action.parameterValues.setPropertyStrategy : undefined;
                    if (!propertyNameChoiceListValuesIndexById[propertyName]) {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById["user-specified-property"],
                            newProp: propertyName,
                            strategy: strategy
                        };
                    } else {
                        formMapping.actions[fieldName] = {
                            fieldName: fieldName,
                            propertyType: propertyNameChoiceListValuesIndexById[propertyName],
                            strategy: strategy
                        };
                    }
                }
            });
            delete formMapping.tempActions;
        };

        this.localToServerFormMapping = function (formMapping) {
            var _formMapping = {
                "metadata": {
                    "id": formMapping.id,
                    "name": formMapping.name,
                    "description": formMapping.description,
                    "scope": formMapping.scope,
                    "tags": ['formMappingRule']
                },

                "condition": {
                    "type": "booleanCondition",
                    "parameterValues": {
                        "operator": "and",
                        "subConditions": []
                    }
                },

                "actions": []
            };

            //conditions
            var pathConditions = [];
            pathConditions.push({
                "parameterValues": {
                    "path": formMapping.primaryPath
                },
                "type": "sourceEventPropertyCondition"
            });
            formMapping.additionalPaths.forEach(function (path) {
                pathConditions.push({
                    "parameterValues": {
                        "path": path
                    },
                    "type": "sourceEventPropertyCondition"
                })
            });

            _formMapping.condition.parameterValues.subConditions.push(
                {
                    "parameterValues": {
                        "formId": formMapping.formId
                    },
                    "type": "formEventCondition"
                },
                {
                    "parameterValues": {
                        "operator": "or",
                        "subConditions": pathConditions
                    },
                    "type": "booleanCondition"
                }
            );

            function prefixPropertyName(name, prefix) {
                return prefix + name + endPrefix;
            }

            //actions
            angular.forEach(formMapping.actions, function(value) {
                value.order = ManagersUtils.formMappingActionStrategies.length - ManagersUtils.formMappingActionStrategies.indexOf(value.strategy);
            });

            angular.forEach(_.sortBy(formMapping.actions, 'order'), function (value) {
                if (value.propertyType) {
                    var strategy = value.strategy;
                    var target = ((value.propertyType.metadata.id == 'user-specified-property' && value.newProp) ? value.newProp : value.propertyType.metadata.id);
                    var action = {
                        "parameterValues": {
                            "setPropertyName": prefixPropertyName(target, setPropertyNamePrefix),
                            "setPropertyStrategy": strategy
                        },
                        "type": "setPropertyAction"
                    };

                    if (value.propertyType.type == "integer") {
                        action.parameterValues.setPropertyValueInteger = prefixPropertyName(value.fieldName, setPropertyValuePrefix);
                    }
                    else {
                        if (value.propertyType.multivalued) {
                            action.parameterValues.setPropertyValueMultiple = prefixPropertyName(value.fieldName, setPropertyValuePrefix);
                        }else{
                            action.parameterValues.setPropertyValue = prefixPropertyName(value.fieldName, setPropertyValuePrefix); 
                        }
                    }
                    _formMapping.actions.push(action);
                }
            });

            //lists
            angular.forEach(formMapping.lists, function (value, key) {
                _formMapping.actions.push({
                    "parameterValues": {
                        "setPropertyName": "systemProperties.lists",
                        "setPropertyValue": value,
                        "setPropertyStrategy": "addValue"
                    },
                    "type": "setPropertyAction"
                });
            });

            return _formMapping;
        };

        this.remove = function (id) {
            this.deletedItem = id;
            return $http.delete(formMappingContext + id);
        };

        this.getFormFactory = function(id) {
            url = ManagersContext.cmsContext + "/modules/formfactory/live/skeleton/" + id + "/" + ManagersContext.jahiaUILocale;
            return $http.get(url);
        }

    }])

    .service('internalSearchService', ['$http', '$q', 'i18nService', 'queryService', function ($http, $q, i18nService, queryService) {
        // todo: replace by ManagerUtils.AggregateDisplay at some point
        var Search = function (searchString, occurrences) {
            var SearchItem = function (value, occurrences, missing) {
                this.item = value;
                this.occurrences = occurrences;
                this.missing = missing;

                this.importance = function (totalOccurrences) {
                    return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
                }
            };

            this.searchString = searchString;
            this.occurrences = occurrences;

            this.origins = [];
            this.setOrigins = function (origins) {
                this.origins = this._setDetails(origins);
            };

            this.userIds = [];
            this.setUserIds = function (userIds) {
                this.userIds = this._setDetails(userIds);
            };

            this.importance = function (totalOccurrences) {
                return ManagersUtils.getPercentage(this.occurrences / totalOccurrences);
            };

            this._setDetails = function (details) {
                if (details) {
                    var detailsArray = [];
                    var totalOccurrences = 0;
                    for (var detail in details) {
                        if (ManagersUtils.acceptProperty(details, detail)) {
                            occurrences = details[detail];

                            var missing = detail == "_missing";
                            if (missing) {
                                // replace _missing by something more user friendly
                                detail = i18nService.message("serverSettings.internalSearches.missing");
                            }
                            
                            totalOccurrences += occurrences;
                            detailsArray.push(new SearchItem(detail, occurrences, missing));
                        }
                    }
                    this.occurrences = totalOccurrences;
                    return detailsArray;
                }
            };

            this.languages = [];
            this.setLanguages = function (lang) {
                this.languages = this._setDetails(lang);
            }
        };

        // todo: extract scope condition to reuse it
        // todo: use ManagerUtils.booleanAnd
        var searchEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "search"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        // todo: use ManagerUtils.booleanAnd
        var viewEventsCondition = {
            condition: {
                type: "booleanCondition",
                parameterValues: {
                    subConditions: [
                        {
                            type: "sessionPropertyCondition",
                            parameterValues: {
                                propertyName: "source.scope",
                                comparisonOperator: "equals",
                                propertyValue: ManagersContext.dmfScope
                            }
                        },
                        {
                            type: "eventTypeCondition",
                            parameterValues: {
                                eventTypeId: "view"
                            }
                        }
                    ],
                    operator: "and"
                }
            }
        };

        this.getViewEventsCondition = function(){
            return viewEventsCondition;
        };


        this.selectSearch = function (searchTerm) {
            this.selectedSearch = searchTerm;
        };

        this.selectUrl = function (urlPath) {
            this.selectedUrl = urlPath;
        };


        this.getSearchOccurrencesByTerm = function () {
            return queryService.query('event', 'properties.keyword', searchEventsCondition);
        };

        this.getSearchOccurrencesByTermAndDate = function (from, to){
            var deferred = $q.defer();
            var searchEventsWithDateRangeCondition = {
                "condition": ManagersUtils.getDateRangeConditionBody(from, to)
            };

            searchEventsWithDateRangeCondition.condition.parameterValues.subConditions.push({
                type: "eventTypeCondition",
                parameterValues: {
                    eventTypeId: "search"
                }
            });

            queryService.query('event', 'properties.keyword', searchEventsWithDateRangeCondition).success(
                function (data) {
                    var searches = [];
                    for (var search in data) {
                        if (ManagersUtils.acceptProperty(data, search)) {
                            searches.push({
                                searchString: search,
                                nbOcc: data[search],
                                importance: ManagersUtils.getPercentage(data[search] / data._filtered)
                            });
                        }
                    }
                    var result = {data: searches};
                    deferred.resolve(result);
                }
            ).error(
                function () {
                    deferred.reject();
                }
            );
            return deferred.promise;
        };

        this.getSearchDetails = function (searchTerm) {
            // if we didn't provide a search term, use the one potentially already selected
            searchTerm = searchTerm || this.selectedSearch;

            var search = new Search(searchTerm, -1);

            var condition = {};
            angular.copy(searchEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "properties.keyword",
                    "comparisonOperator": "equals",
                    "propertyValue": searchTerm
                }
            });

            queryService.query('event', 'properties.origin', condition).success(function (data) {
                search.setOrigins(data);
            });

            queryService.query('event', 'profileId', condition).success(function (data) {
                search.setUserIds(data);
            });

            queryService.query('event', 'properties.language', condition).success(function (data) {
                search.setLanguages(data);
            });

            return search;
        };

        this.getMostVisitedList = function (limit, from, to) {

            var deferred = $q.defer();
            var viewEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    {
                        "type": "eventTypeCondition",
                        "parameterValues": {
                            "eventTypeId": "view"
                        }
                    })
            };

            queryService.query("event", "target.properties.pageInfo.destinationURL", viewEventsByDateCondition).success(function (data) {
                var dataMap = null;
                dataMap = _.map(data, function (occurrences, url) {
                    return {url: url, occurrences: occurrences}
                });
                dataMap.splice(0, 2);

                var result = {data: dataMap, limit: 0};

                if(limit > 0){
                    result.limit = limit;
                }

                //fnCallback(results);
                deferred.resolve(result);

            }).error(function (data, status, headers, config) {
                deferred.reject();
            });
            return deferred.promise;
        };

        this.getMostVisitedDetails = function () {

            var urlPath = this.selectedUrl;

            var search = new Search(urlPath, -1);

            var condition = {};
            angular.copy(viewEventsCondition, condition);
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.destinationURL",
                    "comparisonOperator": "equals",
                    "propertyValue": urlPath
                }
            });


            queryService.query('event', 'profileId', condition).success(function (data) {
                search.setUserIds(data);
            });

            return search;
        }

    }])

    .service('referrerService', ['$http', '$q', 'i18nService', 'queryService', 'profileService', function ($http, $q, i18nService, queryService, profileService) {
        var scopeCondition = {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "scope",
                "comparisonOperator": "equals",
                "propertyValue": ManagersContext.dmfScope
            }
        };

        var viewFromReferrerEventCondition = {
            "type": "eventTypeCondition",
            "parameterValues": {
                "eventTypeId": "viewFromReferrer"
            }
        };

        this.getReferrers = function (from, to) {
            console.log(ManagersContext.proxyServletUrl);
            console.log(ManagersUtils.cxsContext);

            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    viewFromReferrerEventCondition)
            };

            queryService.query('event', 'target.properties.pageInfo.referrerHost', referrerEventsByDateCondition)
            .success(function (data) {
                var referrers = [];
                for (var referrer in data) {
                    if (ManagersUtils.acceptProperty(data, referrer)) {
                        var occurrences = data[referrer];
                        referrers.push({
                            host: referrer,
                            occurrences: occurrences,
                            importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                        });
                    }
                }

                var result = {data: referrers};
                deferred.resolve(result);
            })
            .error(
            function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        this.getReferrerDetails = function (host, offset, length, order) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || this.selectedReferrer;

            var condition = {
                "condition": ManagersUtils.booleanAnd(scopeCondition, {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "properties.referrerHost",
                        "comparisonOperator": "equals",
                        "propertyValue": host
                    }
                }),
                "limit": length,
                "offset": offset,
                "sortby": order,
                "text": null
            };

            return profileService.searchSessions(condition, 'profiles');
        };
        
        this.getReferrerQueries = function (host) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || this.selectedReferrer;

            var referrerQueries = new ManagersUtils.AggregateDisplay(host, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(scopeCondition, viewFromReferrerEventCondition)};
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerHost",
                    "comparisonOperator": "equals",
                    "propertyValue": host
                }
            });

            queryService.query('event', 'target.properties.pageInfo.referrerQuery', condition).success(function (data) {
                referrerQueries.setDetails('queries', data);
            });

            return referrerQueries;
        };

        this.selectReferrer = function(referrer) {
            this.selectedReferrer = referrer;
        };

        var encodeQuery = function (query) {
            return query.replace(/ /g, '+')
        };

        this.decodeQuery = function (query) {
            return query.replace(/\+/g, ' ')
        };

        this.selectQuery = function(query) {
            this.selectedQuery = query;
            return encodeQuery(query);
        };

        this.getQueries = function (from, to) {
            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    viewFromReferrerEventCondition)
            };

            var eventTypeCondition = angular.copy(referrerEventsByDateCondition.condition.parameterValues.subConditions[1]);
            referrerEventsByDateCondition.condition.parameterValues.subConditions[1] = ManagersUtils.booleanAnd(eventTypeCondition, {
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerQuery",
                    "comparisonOperator": "exists"
                }
            });

            queryService.query('event', 'target.properties.pageInfo.referrerQuery', referrerEventsByDateCondition)
                .success(function (data) {
                    var queryList = [];
                    for (var query in data) {
                        if (ManagersUtils.acceptProperty(data, query)) {
                            var occurrences = data[query];
                            queryList.push({
                                referrerQuery: query,
                                occurrences: occurrences,
                                importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                            });
                        }
                    }

                    var result = {data: queryList};
                    deferred.resolve(result);
                })
                .error(function () {
                    deferred.reject();
                });

            return deferred.promise;
        };

        this.getQueryDetails = function (query, offset, length, order) {
            query = this.decodeQuery(query) || this.selectedQuery;

            var condition;
            if ('_missing' === query) {
                condition = {
                    "condition": ManagersUtils.booleanAnd(scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "missing"
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            } else {
                condition = {
                    "condition": ManagersUtils.booleanAnd(scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "equals",
                            "propertyValue": query
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            }

            return profileService.searchSessions(condition, 'profiles');
        };

        this.getQueryReferrers = function (query) {
            // if we didn't provide a search term, use the one potentially already selected
            query = this.decodeQuery(query) || this.selectedQuery;

            var incomingSearch = new ManagersUtils.AggregateDisplay(query, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(scopeCondition, viewFromReferrerEventCondition)};
            if ('_missing' === query) {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "missing"
                    }
                });
            } else {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "equals",
                        "propertyValue": query
                    }
                });
            }

            queryService.query('event', 'target.properties.pageInfo.referrerHost', condition).success(function (data) {
                incomingSearch.setDetails('hosts', data);
            });

            incomingSearch.incomingSearchString = ManagersUtils.humanReadableMissingIfNeeded(query, i18nService);

            return incomingSearch;
        };
    }])

    .service('dataTableService', ['DTOptionsBuilder', 'DTColumnBuilder', 'i18nService',
        function (DTOptionsBuilder, DTColumnBuilder, i18nService) {

        var promiseToCall = null;

        this.setPromise = function(promise){
            promiseToCall = promise;
        };

        this.getPromise = function(){
            return this.promise;
        };

        this.serverData = function(sSource, aoData, fnCallback, oSettings) {
            var draw = _.find(aoData, function (x) {
                return (x.name === 'draw');
            }).value;

            var start = _.find(aoData, function (x) {
                return (x.name === 'start');
            }).value;

            var length = _.find(aoData, function (x) {
                return (x.name === 'length');
            }).value;

            var orderData = _.find(aoData, function (x) {
                return (x.name === "order");
            }).value;

            var order = _.map(orderData, function (o) {
                return {prop: o.column, dir: o.dir};
            })[0];

            promiseToCall.then(
                function(dataMap){

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": dataMap.data.length,
                        "iTotalDisplayRecords": dataMap.data.length,
                        "aaData": dataMap.data
                    };
                    if(order) {
                        results.aaData = _.sortBy(dataMap.data, _.keys(dataMap.data[0])[order.prop]);
                        if(order.dir === 'desc'){
                            results.aaData = results.aaData.reverse();
                        }
                    }
                    if(dataMap.limit > 0){
                        results.aaData.splice(dataMap.limit);
                    }else{
                        results.aaData = results.aaData.splice(start, length);
                    }

                    fnCallback(results);
                },
                function(){
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                }
            );
        };

        this.generateDataTableOptions = function (options) {

            var dtOptions = DTOptionsBuilder.newOptions();

            dtOptions.withOption('lengthMenu', options.lengthMenu ? options.lengthMenu : [ 25, 50, 100 ])
                .withOption('pageLength', options.pageLength ? options.pageLength : 25);

            if(options.rowCallback){
                dtOptions = dtOptions.withOption('fnRowCallback',
                    options.rowCallback);
            }
            if(options.createdRow){
                dtOptions = dtOptions.withOption('createdRow',
                    options.createdRow);
            }
            dtOptions = dtOptions.withOption('processing', options.processing ? options.processing : true)
                .withOption('serverSide', options.serverSide ? options.serverSide : true)
                .withOption('sDom', 'lt')
                .withFnServerData(this.serverData)
                .withOption('order', options.defaultOrder);
            if(options.paging != undefined){
                dtOptions = dtOptions.withOption('paging', false);
            }
            if(options.ordering != undefined){
                dtOptions = dtOptions.withOption('ordering', false);
            }
            return dtOptions;

        };

        this.generateDataTableColumns = function (columns) {
            var dtColumns = [];
            _.forEach(columns, function(column){
                var columnItem = DTColumnBuilder.newColumn(column.name).withTitle(i18nService.message(column.bundleKey));
                if(column.notSortable){
                    columnItem = columnItem.notSortable();
                }
                if(column.actionsHtml){
                    columnItem = columnItem.renderWith(column.actionsHtml);
                }
                if(column.options && column.options.length){
                    _.forEach(column.options, function(columnOption){
                        columnItem = columnItem.withOption(columnOption[0], columnOption[1]);
                    });
                }
                dtColumns.push(columnItem);
            });
            return dtColumns;
        };

    }])
    .service('geonamesService', ['$http', function ($http) {
        this.getEntries = function () {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/entries';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    url = url + ("/" + arguments[i]);
                }
            }
            if (length == url.length) {
                url += "/";
            }

            return $http.get(url);
        };

        this.getHierarchy = function (administrativeSbudivision) {
            return $http.get(ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/hierarchy/' + administrativeSbudivision);
        };

        this.getCities = function () {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/cities';
            var length = url.length;
            if (arguments.length > 0) {
                for (var i = 0; i < arguments.length; i++) {
                    if (arguments[i]) {
                        url = url + ("/" + arguments[i]);
                    }
                }
            }
            if (length == url.length) {
                url += "/";
            }
            return $http.get(url);
        };

        this.getCapital = function (itemId) {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/capitals/' + itemId;
            return $http.get(url);
        };

        this.reverseGeoCode = function (lat, lon) {
            var url = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext + 'geonames/reverseGeoCode/' + lat + ',' + lon;
            return $http.get(url);
        }
    }]);

